package com.persagy.apm.report.common.schedule;


import com.persagy.apm.report.common.utils.DateUtils;
import com.persagy.apm.report.dependencies.saasweb.model.vo.GroupCodeVO;
import com.persagy.apm.report.dependencies.saasweb.service.ISaasWebService;
import com.persagy.apm.report.outline.constants.enums.ReportStateEnum;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.outline.model.dto.QueryReportOutlineDTO;
import com.persagy.apm.report.outline.service.IReportOutlineService;
import com.persagy.framework.tenant.mybatis.datasource.db.holder.SaasContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 定时更新还未生成成功的报告状态为生成失败
 *
 * @author lixing
 * @version V1.0 2021/8/9 11:59 上午
 **/
@Component
@Slf4j
public class ReportGenerateFailScheduleTask {
    @Autowired
    IReportOutlineService reportOutlineService;
    @Value("${maxGenerateMinutes:5}")
    private int maxGenerateMinutes;
    @Value("${failReportPeriod:600000}")
    private String failReportPeriod;
    
    @Autowired
    private ISaasWebService saasWebService;

    /**
     * 定时更新还未生成成功的报告状态为生成失败（目的是为了处理宕机引起的报告状态一直处于生成中的问题）
     *
     * @author lixing
     * @version V1.0 2021/8/9 12:01 下午
     */
    @Scheduled(fixedRateString = "${failReportPeriod:600000}")
    public void checkAndFailReportGenerateState() {
        log.info("开始检查生成时间超过{}ms, 依然处于生成中的报告", failReportPeriod);
        List<GroupCodeVO> groupList = saasWebService.getGroupList();
        for (GroupCodeVO groupCodeVO : groupList) {
            SaasContext.setContextGroupCode(groupCodeVO.getCode());
            try {
                queryAndFailReports();
            } catch (Exception e) {
                log.error("{}集团更新未生成成功的报告状态为生成失败的定时任务执行失败", groupCodeVO.getCode(), e);
            }
        }

    }

    /**
     * 定时更新还未生成成功的报告状态为生成失败
     * 根据线程中的集团编码更新对应集团的数据，一次只能更新一个集团的数据
     *
     * @author lixing
     * @version V1.0 2021/8/9 12:01 下午
     */
    private void queryAndFailReports() {
        // 查询生成状态为生成中的报告
        QueryReportOutlineDTO queryReportOutlineDTO = new QueryReportOutlineDTO();
        queryReportOutlineDTO.setStatus(ReportStateEnum.BUILDING.getType());
        List<ReportOutline> reportOutlines = reportOutlineService.queryReportOutlineList(queryReportOutlineDTO);
        // 过滤当前时间 - 更新时间 > failReportPeriod的报告
        if (CollectionUtils.isEmpty(reportOutlines)) {
            return;
        }

        Date now = new Date();
        List<ReportOutline> reports2Fail = reportOutlines.stream().filter(reportOutline -> {
            // 获取两个时间点相差的毫秒数
            Double dateDuration = DateUtils.getDateDurationMs(reportOutline.getModifiedTime(), now);
            log.info( "{}生成时间：{}min", reportOutline.getName(),
                    DateUtils.getDateDurationMin(reportOutline.getModifiedTime(), now));
            return dateDuration > maxGenerateMinutes * 60 * 1000;
        }).collect(Collectors.toList());
        log.info("共有{}个报告生成超时", reports2Fail.size());
        // 将这些报告的状态置为生成失败
        reports2Fail.forEach(reportOutline -> {
            reportOutline.setStatus(ReportStateEnum.BUILD_FAILED.getType());
            reportOutlineService.updateReportOutline(reportOutline);
        });
        log.info("已将这些报告的状态更新为生成失败");
    }
}
